import * as React from 'react';
import { Box, Typography, TextField, Button, Select, MenuItem, Snackbar } from '@mui/material';
import QRCode from 'react-qr-code';
import { ContactPerson, Address, Dewar } from "../../openapi";

interface DewarDetailsProps {
    dewar: Dewar;
    trackingNumber: string;
    setTrackingNumber: React.Dispatch<React.SetStateAction<string>>;
    contactPersons: ContactPerson[];
    returnAddresses: Address[];
}


const DewarDetails: React.FC<DewarDetailsProps> = ({
                                                       dewar,
                                                       trackingNumber,
                                                       setTrackingNumber,
                                                       contactPersons,
                                                       returnAddresses
                                                   }) => {
    const [selectedContactPerson, setSelectedContactPerson] = React.useState<string>(contactPersons[0]?.firstname || '');
    const [selectedReturnAddress, setSelectedReturnAddress] = React.useState<string>(returnAddresses[0]?.id?.toString() || '');

    const updateSelectedDetails = (contactPerson?: { firstname: string }, returnAddress?: Address) => {
        if (contactPerson) setSelectedContactPerson(contactPerson.firstname);
        if (returnAddress?.id != null) {
            setSelectedReturnAddress(returnAddress.id.toString());
        };
    };

    React.useEffect(() => {
        updateSelectedDetails(contactPersons[0], returnAddresses[0]);
    }, [contactPersons, returnAddresses]);

    const [newContactPerson, setNewContactPerson] = React.useState<string>('');
    const [newReturnAddress, setNewReturnAddress] = React.useState<string>('');
    const [feedbackMessage, setFeedbackMessage] = React.useState<string>('');
    const [openSnackbar, setOpenSnackbar] = React.useState<boolean>(false);

    // Ensure dewar is defined before attempting to render the dewar details
    if (!dewar) {
        return <Typography>No dewar selected.</Typography>;
    }

    const handleAddContact = () => {
        if (newContactPerson.trim() === '') {
            setFeedbackMessage('Please enter a valid contact person name.');
        } else {
            setNewContactPerson(''); // Add logic to save the new contact person
            setFeedbackMessage('Contact person added successfully.');
        }
        setOpenSnackbar(true);
    };

    const handleAddAddress = () => {
        if (newReturnAddress.trim() === '') {
            setFeedbackMessage('Please enter a valid return address.');
        } else {
            setNewReturnAddress(''); // Add logic to save the new return address
            setFeedbackMessage('Return address added successfully.');
        }
        setOpenSnackbar(true);
    };

    return (
        <Box sx={{ marginTop: 2 }}>
            <Typography variant="h6">Selected Dewar: {dewar.dewar_name}</Typography>

            <TextField
                label="Tracking Number"
                value={trackingNumber}
                onChange={(e) => setTrackingNumber(e.target.value)}
                variant="outlined"
                sx={{ width: '300px', marginBottom: 2 }}
            />

            {/* QR Code display */}
            <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 2 }}>
                <Box sx={{ width: 80, height: 80, backgroundColor: '#e0e0e0', display: 'flex', alignItems: 'center', justifyContent: 'center' }}>
                    {dewar.qrcode ? (
                        <QRCode value={dewar.qrcode} size={70} />
                    ) : (
                        <Typography>No QR code available</Typography>
                    )}
                </Box>
                <Button variant="contained" onClick={() => { /** Add logic to generate QR Code */ }}>
                    Generate QR Code
                </Button>
            </Box>

            <Typography variant="body1">Number of Pucks: {dewar.number_of_pucks}</Typography>
            <Typography variant="body1">Number of Samples: {dewar.number_of_samples}</Typography>

            {/* Dropdown for Contact Person */}
            <Typography variant="body1">Current Contact Person:</Typography>
            <Select
                value={selectedContactPerson}
                onChange={(e) => setSelectedContactPerson(e.target.value)}
                displayEmpty
                fullWidth
                sx={{ marginBottom: 2 }}
                variant={'outlined'}
            >
                <MenuItem value="" disabled>Select Contact Person</MenuItem>
                {contactPersons.map((person) => (
                    <MenuItem key={person.id} value={person.firstname}>{person.firstname + " " + person.lastname}</MenuItem>
                ))}
                <MenuItem value="add">Add New Contact Person</MenuItem>
            </Select>
            {selectedContactPerson === "add" && (
                <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 2 }}>
                    <TextField
                        label="New Contact Person"
                        value={newContactPerson}
                        onChange={(e) => setNewContactPerson(e.target.value)}
                        variant="outlined"
                        sx={{ marginRight: 1, flexGrow: 1 }}
                    />
                    <Button variant="contained" onClick={handleAddContact}>
                        Add
                    </Button>
                </Box>
            )}

            {/* Dropdown for Return Address */}
            <Typography variant="body1">Current Return Address:</Typography>
            <Select
                value={selectedReturnAddress}
                onChange={(e) => setSelectedReturnAddress(e.target.value)}
                displayEmpty
                fullWidth
                sx={{ marginBottom: 2 }}
                variant={'outlined'}
            >
                <MenuItem value="" disabled>Select Return Address</MenuItem>
                {returnAddresses.map((address) => (
                    <MenuItem key={address.id ?? 'unknown'} value={address.id?.toString() ?? 'unknown'}>
                        {address.street} </MenuItem>
                ))}
                <MenuItem value="add">Add New Return Address</MenuItem>
            </Select>
            {selectedReturnAddress === "add" && (
                <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 2 }}>
                    <TextField
                        label="New Return Address"
                        value={newReturnAddress}
                        onChange={(e) => setNewReturnAddress(e.target.value)}
                        variant="outlined"
                        sx={{ marginRight: 1, flexGrow: 1 }}
                    />
                    <Button variant="contained" onClick={handleAddAddress}>
                        Add
                    </Button>
                </Box>
            )}

            {/* Snackbar for feedback messages */}
            <Snackbar
                open={openSnackbar}
                autoHideDuration={6000}
                onClose={() => setOpenSnackbar(false)}
                message={feedbackMessage}
            />
        </Box>
    );
};

export default DewarDetails;